package de.fraunhofer.iem.dataprovider.tools.ort.service

import de.fraunhofer.iem.dataprovider.configuration.OpenCodeApiProperties
import de.fraunhofer.iem.dataprovider.kpi.dto.KPICreateDto
import de.fraunhofer.iem.dataprovider.kpi.strategy.RawValueKPICalculationStrategy
import de.fraunhofer.iem.dataprovider.tool.dto.CreateToolDto
import de.fraunhofer.iem.dataprovider.tool.entity.ToolEntity
import de.fraunhofer.iem.dataprovider.tool.service.ToolService
import de.fraunhofer.iem.dataprovider.tools.ort.dto.VulnerabilityDto
import de.fraunhofer.iem.dataprovider.tools.ort.json.OrtJson
import io.ktor.client.*
import io.ktor.client.call.*
import io.ktor.client.engine.cio.*
import io.ktor.client.plugins.contentnegotiation.*
import io.ktor.client.request.*
import io.ktor.client.statement.*
import io.ktor.serialization.kotlinx.json.*
import kotlinx.serialization.json.Json
import org.springframework.stereotype.Service

@Service
class OrtService(private val openCodeApiProperties: OpenCodeApiProperties, private val toolService: ToolService) {

    val client = HttpClient(CIO) {
        install(ContentNegotiation) {
            json(
                Json { ignoreUnknownKeys = true }
            )
        }
    }

    val toolEntity: ToolEntity = getOrCreateToolEntity()

    private final fun getOrCreateToolEntity(): ToolEntity {
        val createToolDto = CreateToolDto("ORT")
        return toolService.findOrCreateTool(createToolDto)
    }

    suspend fun getOrtResults(repoId: Long): List<VulnerabilityDto> {
        val response: HttpResponse = client.get(getToolApiPath(repoId))
        val ortJson = response.body<OrtJson>()
        client.close()

        if (ortJson.code == 200) {
            return ortJson.result.mapNotNull {
                if (it.cveId != null && it.packageName != null && it.severity != null) {
                    VulnerabilityDto(it.cveId, it.packageName, it.severity)
                } else {
                    null
                }
            }
        }
        return emptyList()
    }

    fun calculateVulnerabilityKpis(vulnerabilityDtos: List<VulnerabilityDto>): List<KPICreateDto> {
        return vulnerabilityDtos.map {
            KPICreateDto(
                "Vulnerability ${it.cveIdentifier}",
                "Affected package: ${it.packageName}",
                false,
                RawValueKPICalculationStrategy((it.severity * 10).toInt()),
                displayScore = it.severity.toString()
            )
        }
    }

    private fun getToolApiPath(repoId: Long): String {
        return "${openCodeApiProperties.basePath}106${openCodeApiProperties.ort}"
    }

}
